#include "LO_exact.h"
#include <cmath>
#include "shared.h"
#include "phase2.h"
#include "../src/params.h"
#include "../src/def_params.h"
#include "clooptools.h"

double LO_exact(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch, bool check_mzh_cutoff){
	double cos = 2.*xs[0]-1.;
	if(abs(cos) > 0.99999999){
		//TL (14/09/16):
		//Introduced this cutoff to prevent LoopTools errors, should not impact precision
		return 0;
	}
	double jacobian = 2.;
	double z2 = 1.6449340668482264365;
	double mh2 = mh*mh;
	double mz2 = mz*mz;
	double GF = alpha*Pi/sqrt(2.)/(mz*mz*cw2*sw2);
	double s = sqrs*sqrs;
	if(check_mzh_cutoff && use_inv_mass_cutoff && s > inv_mass_cutoff*inv_mass_cutoff) return 0;
	//Set mu^2 = s = (p_Z+p_H)^2 or fixed value
	double mu2_R = mu_var_R * mu_var_R, mu2_F = mu_var_F * mu_var_F;
	if(mu_type == 0){
		mu2_R *= fixed_mu_R * fixed_mu_R;
		mu2_F *= fixed_mu_F * fixed_mu_F;
	}
	else{
		mu2_R *= s;
		mu2_F *= s;
	}
	double lnmt = log(mt*mt/mu2_R);
	double kaellenroot = sqrt(kaellen_function(s,mz2,mh2));
	double pref = phase2prefactor(s,mz,mh);
	if(pref == 0) return 0;
	double flux = 1./s/2.;
	double sp12 = s/2.;
	double sp13 = 1./4.*(-(s+mz2-mh2) - kaellenroot*cos);
	double sp23 = 1./4.*(-(s+mz2-mh2) + kaellenroot*cos);
	double alphas = ch->get_alpha_s(mu2_R);
	double T;
	
	//Check cut on Higgs transversal momentum
	if(use_pth_min){
		double pth = sqrt(4/s*sp13*sp23-mz*mz);
		if(pth < pth_min) return 0;
	}

// cut out nan-region
        if(2*sp13*sp23 - sp12*(mz)*(mz) == 0.){
		if(ch->dropcount!=nullptr) ++*(ch->dropcount);
		return 0;
	}

// set LoopTools parameters
	setmudim(mu2_R);
	setlambda(ltlambda);

	#include "LO_exact.ampsq"
	double ampsq = pref*flux*jacobian*T;
	return ampsq * get_pdfs(mu2_F);
}

double LO_exact(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return LO_exact(xs, sqrs, get_pdfs, ch, true);
}
