#ifndef NLO_DIFFERENTIAL_H_
#define NLO_DIFFERENTIAL_H_


double NLO_differential_phase2(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	
	double lo_exact = LO_exact(xs, sqrs, get_pdfs, ch);
	
	if(lo_exact == 0) return 0;
	
	double numer = 0, denom = 0;
	
	for(unsigned int inv_order = 0; inv_order <= diff_order; inv_order += 2){
		numer += NLO_phase2(xs, sqrs, get_pdfs, ch, inv_order);
		denom += LO(xs, sqrs, get_pdfs, ch, inv_order);
	} 
	
	return lo_exact * numer / denom;
}


double NLO_differential_phase2eta(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	
	double lo_exact = LO_exact(xs, sqrs, get_pdfs, ch, false);
	
	if(lo_exact == 0) return 0;
	
	double numer = 0, denom = 0;
	
	for(unsigned int inv_order = 0; inv_order <= diff_order; inv_order += 2){
		numer += NLO_phase2eta(xs, sqrs, get_pdfs, ch, inv_order);
		denom += LO(xs, sqrs, get_pdfs, ch, inv_order, false);
	} 
	
	return lo_exact * numer / denom;
}


double NLO_differential_reducible(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	
	double lo_exact = LO_exact(xs, sqrs, get_pdfs, ch);
	
	if(lo_exact == 0) return 0;
	
	double numer = 0, denom = 0;
	
	for(unsigned int inv_order = 0; inv_order <= diff_order; inv_order += 2){
		numer += NLO_reducible(xs, sqrs, get_pdfs, ch, inv_order);
		denom += LO(xs, sqrs, get_pdfs, ch, inv_order);
	} 
	
	return lo_exact * numer / denom;
}




#endif /* NLO_DIFFERENTIAL_H_ */






