#include "NLO_phase2.h"
#include "phase2.h"
#include "../src/params.h"
#include "../src/def_params.h"
#include "../src/channel.h"


double NLO_phase2_ampsq(double sp12, double sp13, double sp23, double alphas, double musquare, unsigned int inv_mt_pow){
	switch(inv_mt_pow){
		case 0:
		{
			#include "NLO_phase2_0.ampsq"
			return ampsq;
		}
		case 2:
		{
			#include "NLO_phase2_2.ampsq"
			return ampsq;
		}
		case 4:
		{
			#include "NLO_phase2_4.ampsq"
			return ampsq;
		}
		case 6:
		{
			#include "NLO_phase2_6.ampsq"
			return ampsq;
		}
		case 8:
		{
			#include "NLO_phase2_8.ampsq"
			return ampsq;
		}
		default:
			cerr << "No NLO phase2 amplitude squared defined for order 1/m_t^ " << inv_mt_pow << "." << endl;
			exit(1);	
	}
	return 0;
}




double NLO_phase2(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch, unsigned int inv_mt_pow){
	if(use_mt_threshold && inv_mt_pow > 0 && sqrs > mt_threshold) return 0;
	double s = sqrs*sqrs;
	double pref = phase2prefactor(s,mz,mh)/s/2;
	
	if(pref == 0) return 0;
	
	if(use_inv_mass_cutoff && s > inv_mass_cutoff*inv_mass_cutoff) return 0;
	//Set mu^2 = s = (p_Z+p_H)^2 or fixed value
	double mu2_R = mu_var_R * mu_var_R, mu2_F = mu_var_F * mu_var_F;
	if(mu_type == 0){
		mu2_R *= fixed_mu_R * fixed_mu_R;
		mu2_F *= fixed_mu_F * fixed_mu_F;
	}
	else{
		mu2_R *= s;
		mu2_F *= s;
	}
	
	pref *= 2; //Factor from measure, dcos -> 2*dx

	double cos = 2*xs[0]-1;
	double sp12 = s/2;
	double kaellenroot = sqrt(kaellen_function(s,mz*mz,mh*mh));
	double sp13 = -(s+mz*mz-mh*mh + kaellenroot*cos)/4;
	double sp23 = -(s+mz*mz-mh*mh - kaellenroot*cos)/4;
	
	//Check cut on Higgs transversal momentum
	if(use_pth_min){
		double pth = sqrt(4/s*sp13*sp23-mz*mz);
		if(pth < pth_min) return 0;
	}
	
	double ampsq = NLO_phase2_ampsq(sp12, sp13, sp23, ch->get_alpha_s(mu2_R), mu2_R, inv_mt_pow);
	
	ampsq /= 256; //Averaging factor for 2 incoming gluons. 256 = 2^2 (polarisation) * 8^2 (colour)
	
	return pref*ampsq/pow(mt,inv_mt_pow)*get_pdfs(mu2_F);
}



double NLO_phase2_0(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase2(xs, sqrs, get_pdfs, ch, 0);
}

double NLO_phase2_2(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase2(xs, sqrs, get_pdfs, ch, 2);
}

double NLO_phase2_4(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase2(xs, sqrs, get_pdfs, ch, 4);
}

double NLO_phase2_6(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase2(xs, sqrs, get_pdfs, ch, 6);
}

double NLO_phase2_8(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase2(xs, sqrs, get_pdfs, ch, 8);
}

#include "../src/undef_params.h"
