#include "LO.h"
#include "NLO_phase2eta.h"
#include "phase2.h"
#include "../src/params.h"
#include "../src/def_params.h"
#include "../src/channel.h"



double NLO_phase2eta_ampsq(double sp12, double sp13, double sp23, double eta, double alphas, double alphasx, double musquare, double musquarex, unsigned int inv_mt_pow){
		switch(inv_mt_pow){
		case 0:
		{
			#include "NLO_phase2eta_0.ampsq"
			return ampsq;
		}
		case 2:
		{
			return 0;
		}
		case 4:
		{
			#include "NLO_phase2eta_4.ampsq"
			return ampsq;
		}
		case 6:
		{
			#include "NLO_phase2eta_6.ampsq"
			return ampsq;
		}
		case 8:
		{
			#include "NLO_phase2eta_8.ampsq"
			return ampsq;
		}
		default:
			cerr << "No NLO phase2eta amplitude squared defined for order 1/m_t^ " << inv_mt_pow << "." << endl;
			exit(1);	
	}
	return 0;
}


double NLO_phase2eta(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch, unsigned int inv_mt_pow){
	if(use_mt_threshold && inv_mt_pow > 0 && sqrs > mt_threshold) return 0;
	double s = sqrs*sqrs;
	double eta = xs[1];
	double cos = 2*xs[0]-1;
	if(eta < etadel || 1-eta < etadel) return 0;

	double sx = s * (1-eta);

	//Flux factor 1/2s
	//Factor 2 from measure, dcos -> 2*dx
	double pref = phase2prefactor(s,mz,mh)/s;
	double prefx = phase2prefactor(sx,mz,mh)/sx;
	
	double total = 0;
	
	if(pref != 0){		
		double sp12 = s/2;
		double kaellenroot = sqrt(kaellen_function(s,mz*mz,mh*mh));
		double sp13 = -(s+mz*mz-mh*mh + kaellenroot*cos)/4;
		double sp23 = -(s+mz*mz-mh*mh - kaellenroot*cos)/4;
		//Check cut on Higgs transversal momentum
		bool add_contribution = true;
		if(use_pth_min){
			double pth = sqrt(4/s*sp13*sp23-mz*mz);
			if(pth < pth_min) add_contribution = false;
		}
		if(use_inv_mass_cutoff){
			if(s > inv_mass_cutoff*inv_mass_cutoff) add_contribution = false;
		}
		if(add_contribution){
			//Set mu^2 = s, (p_Z+p_H)^2 or fixed value
			double mu2_R = mu_var_R * mu_var_R, mu2_F = mu_var_F * mu_var_F;
			if(mu_type == 0){
				mu2_R *= fixed_mu_R * fixed_mu_R;
				mu2_F *= fixed_mu_F * fixed_mu_F;
			}
			else{
				mu2_R *= s;
				mu2_F *= s;
			}
			double alphas = ch->get_alpha_s(mu2_R);
			double LO0  = LO_ampsq(inv_mt_pow, sp12 , sp13 , sp23 , alphas )*alphas /Pi;
			double plus0 = 2*log(eta)/eta - log(mu2_R /s)/eta;
			double pgg0 = 6;
			total -= pref*LO0*pgg0*plus0 * get_pdfs(mu2_F);
		}
	}
	
	if(prefx != 0){
		double sp12 = sx/2;
		double kaellenroot = sqrt(kaellen_function(sx,mz*mz,mh*mh));
		double sp13 = -(sx+mz*mz-mh*mh + kaellenroot*cos)/4;
		double sp23 = -(sx+mz*mz-mh*mh - kaellenroot*cos)/4;
		//Check cut on Higgs transversal momentum
		bool add_contribution = true;
		if(use_pth_min){
			double pth = sqrt(4/sx*sp13*sp23-mz*mz);
			if(pth < pth_min) add_contribution = false;
		}
		if(use_inv_mass_cutoff){
			if(sx > inv_mass_cutoff*inv_mass_cutoff) add_contribution = false;
		}
		if(add_contribution){
			//Set mu^2 = s, (p_Z+p_H)^2 or fixed value
			double mu2_R = mu_var_R * mu_var_R, mu2_F = mu_var_F * mu_var_F;
			if(mu_type == 0){
				mu2_R *= fixed_mu_R * fixed_mu_R;
				mu2_F *= fixed_mu_F * fixed_mu_F;
			}
			else if(mu_type == 1){
				mu2_R *= s;
				mu2_F *= s;
			}
			else{
				mu2_R *= sx;
				mu2_F *= sx;			
			}
			double alphas = ch->get_alpha_s(mu2_R);
			double LOx  = LO_ampsq(inv_mt_pow, sp12 , sp13 , sp23 , alphas )*alphas /Pi;
			double plusx = 2*log(eta)/eta - log(mu2_R/s)/eta;
			double pggx = eta * (eta/(1-eta) + (1-eta)/eta + eta*(1-eta)) * 6;
			total += prefx*LOx*pggx*plusx * get_pdfs(mu2_F);
		}
	}
	
	total /= 256; //Averaging factor for 2 incoming gluons. 256 = 2^2 (polarisation) * 8^2 (colour)
	
	return total/pow(mt,inv_mt_pow);
}


double NLO_phase2eta_0(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase2eta(xs, sqrs, get_pdfs, ch, 0);
}

double NLO_phase2eta_2(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase2eta(xs, sqrs, get_pdfs, ch, 2);
}

double NLO_phase2eta_4(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase2eta(xs, sqrs, get_pdfs, ch, 4);
}

double NLO_phase2eta_6(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase2eta(xs, sqrs, get_pdfs, ch, 6);
}

double NLO_phase2eta_8(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase2eta(xs, sqrs, get_pdfs, ch, 8);
}


#include "../src/undef_params.h"
