#include "NLO_phase3.h"
#include <cmath>
#include "shared.h"
#include "phase3.h"
#include "LO.h"
#include "../src/params.h"
#include "../src/def_params.h"


using namespace std;


//gg splitting function for z < 1
double splitting_gg(double z){
	return 2 * 3 * (z/(1-z)+(1-z)/z + z*(1-z));
}

//NLO real correction collinear limit (costg -> +/-1) of eta^2*(1-costg^2) |M|^2
double NLO_collinear_limit_ampsq(unsigned int inv_mt_pow, double eta, double pz, double Ez, double costz, double costg, double sqrs, double alphas){
	double s = sqrs*sqrs*(1-eta);
	double sp12 = s/2;
	double kaellenroot = sqrt(kaellen_function(s,mz*mz,mh*mh));
	double sp13 = -(s+mz*mz-mh*mh + kaellenroot*costg)/4;
	double sp23 = -(s+mz*mz-mh*mh - kaellenroot*costg)/4;
	
    double ampsq_LO = LO_ampsq(inv_mt_pow, sp12, sp13, sp23, alphas);
      
	return ampsq_LO * 32*Pi * alphas / sqrs/sqrs * eta/(1-eta) * splitting_gg(1-eta);
}

//NLO real correction soft limit (eta -> 0) of eta^2*(1-costg^2) |M|^2
double NLO_soft_limit_ampsq(unsigned int inv_mt_pow, double pz, double Ez, double costz, double sqrs, double alphas){
	double sp12 = sqrs*sqrs/2;
	double sp13 = -sqrs/2 * (Ez - pz*costz);
	double sp23 = -sqrs/2 * (Ez + pz*costz);
	
    double ampsq_LO = LO_ampsq(inv_mt_pow, sp12, sp13, sp23, alphas);
      
	return ampsq_LO * 64*Pi * alphas *3 / sqrs/sqrs;
}


//Full NLO real correction value of eta^2*(1-costg^2) |M|^2
double NLO_phase3_full_ampsq(unsigned int inv_mt_pow, double eta, double pz,double Ez,double costz,double costg,double costzg, double sqrs, double alphas){

	double sp12 = sqrs*sqrs/2;
	double sp13 = -sqrs*sqrs/4 * eta *(1 - costg);
	double sp14 = -sqrs/2 * (Ez - pz*costz);
	double sp23 = -sqrs*sqrs/4 * eta *(1 + costg);
	double sp24 = -sqrs/2 * (Ez + pz*costz);
	double sp34 = sqrs/2 * eta *(Ez - pz * costzg);
	
	
	double mu = mu_var_R;
	if(mu_type == 0){
		mu *= fixed_mu_R;
	}
	else if(mu_type == 1){
		mu *= sqrs;
	}
	else{		
		mu *= sqrt(1-eta)*sqrs;
	}

	switch(inv_mt_pow){
		case 0:
		{
			#include "NLO_phase3_full_0.ampsq"
			//#include "zhrealsq_simpa.inc"
			return ampsq;
		}
		case 2:
			return 0;
		case 4:
		{
			#include "NLO_phase3_full_4.ampsq"
			//#include "zhrealsq_simpa_M1-4.inc"
			return ampsq;
		}
		case 6:
		{
			#include "NLO_phase3_full_6.ampsq"
			//#include "zhrealsq_simpa_M1-6.inc"
			return ampsq;
		}
		case 8:
		{
			#include "NLO_phase3_full_8.ampsq"
			return ampsq;
		}
		
		default:
			cerr << "No full NLO real amplitude squared defined for order 1/m_t^ " << inv_mt_pow << "." << endl;
			exit(1);
	}

	return 0;
}

double NLO_phase3_alt(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch, unsigned int inv_mt_pow){
// numbering 
// ---------
//   vh@nnlo | this file | kinem.
//   --------+-----------+---------
//     x3    |  xs[0]    | (1-eta)
//     x4    |  xs[1]    | (1-coskappa)/2; kappa = angle between Z and g in the CM system of the Z-H pair
//     x5    |  xs[2]    | (1-costheta)/2  theta = polar angle of the gluon momentum p3
//     x6    |  xs[3]    | (1-cosphi)/2;     phi = azimuthal angle of Z in the CM system of the Z-H pair
// 
// incoming gluons: p1=(E1,0,0,E1), p2=(E2,0,0,-E2)
// outgoing gluon: p3=(E3,0.,E3*sintheta,E3*costheta);
//  E1+E2 = 2*E1 = sqrs
//  


	if(use_mt_threshold && inv_mt_pow > 0 && sqrs > mt_threshold) return 0;

	//Kinematic constraint
	if (sqrs < (mz + mh)) return 0;
	
	//Ensure xs[3] != 0 since phase3alt divides by sin(phi)
	if(xs[3] == 0) return 0;

	double p1,p2,E1,E3,cost1,cost2,cost12;
	double s = sqrs*sqrs;

	double pref_full = phase3alt(xs[0],xs[1],xs[2],xs[3],mz,mh,sqrs, p1,p2,E1,E3,cost1,cost2,cost12);
	
	double eta = 1-xs[0];
	
	double gluon_pt = sqrs*eta/2*sqrt(1-cost2*cost2);
	if(gluon_pt < 1) return 0;

	if ( s * (1-eta) < (mz + mh)*(mz+mh)){
		return 0;
	}
	
	if ( (eta < etadel)    || (eta   > (1.-etadel) ) ) return 0;
	if ( (cost2 < -1.+ydel) || (cost2 > (1.-ydel) ) ) return 0;
	if ( (cost1 < -1.+ydel) || (cost1 > (1.-ydel) ) ) return 0;
	
	double total = 0;
	
	
	bool contrib_with_eta = (!use_inv_mass_cutoff || s*(1-eta) <= inv_mass_cutoff*inv_mass_cutoff);
	bool contrib_without_eta = (!use_inv_mass_cutoff || s <= inv_mass_cutoff*inv_mass_cutoff);


	//Plus distribution over eta: eta=0 means xs[0]=1 (i.e. p2=0)
	//Plus distribution over y = cos(theta_gluon): y=2*xs[2]-1
	
	
	if(contrib_with_eta){//eta != 0
		double fakecos;
		//Set mu^2 = s, (p_Z+p_H)^2 or fixed value
		double mu2_R = mu_var_R * mu_var_R, mu2_F = mu_var_F * mu_var_F;
		if(mu_type == 0){
			mu2_R *= fixed_mu_R * fixed_mu_R;
			mu2_F *= fixed_mu_F * fixed_mu_F;
		}
		else if(mu_type == 1){
			mu2_R *= s;
			mu2_F *= s;
		}
		else{
			mu2_R *= s*(1-eta);
			mu2_F *= s*(1-eta);			
		}
		double alphas = ch->get_alpha_s(mu2_R);
		
		double ampsqfull = 0, ampsqy1 = 0, ampsqym1 = 0;
		double pth2; //=transversal Higgs momentum squared, check the cut for each contribution
		
		pth2 = s/4*eta*eta * (1-cost2*cost2) + sqrs*eta*p1*(cost12 - cost1*cost2) + p1*p1*(1-cost1*cost1);
		if(!use_pth_min || pth2 > pth_min*pth_min) ampsqfull    = NLO_phase3_full_ampsq(inv_mt_pow, eta, p1, E1, cost1, cost2, cost12, sqrs, alphas);
		//ME2test = ampsqfull*pref_full * (1/(1-cost2) + 1/(1+cost2)) / (256*2*s*2*eta*eta);

		double pref_y1 =  phase3alt(xs[0],xs[1],1,xs[3],mz,mh,sqrs, p1,p2,E1,E3,cost1,fakecos,cost12);
		pth2 = p1*p1*(1-cost1*cost1);
		if(!use_pth_min || pth2 > pth_min*pth_min) ampsqy1      = NLO_collinear_limit_ampsq(inv_mt_pow, eta, p1,E1,cost1,  1, sqrs, alphas);
		double pref_ym1 = phase3alt(xs[0],xs[1],0,xs[3],mz,mh,sqrs, p1,p2,E1,E3,cost1,fakecos,cost12);
		pth2 = p1*p1*(1-cost1*cost1);
		if(!use_pth_min || pth2 > pth_min*pth_min) ampsqym1     = NLO_collinear_limit_ampsq(inv_mt_pow, eta, p1,E1,cost1, -1, sqrs, alphas);
		total += get_pdfs(mu2_F)*((ampsqfull*pref_full-ampsqy1*pref_y1)/(1-cost2) + (ampsqfull*pref_full-ampsqym1*pref_ym1)/(1+cost2));
	}
	
	if(contrib_without_eta){//eta == 0, but use etadel instead of 0
		//Set mu^2 = s, (p_Z+p_H)^2 or fixed value
		double mu2_R = mu_var_R * mu_var_R, mu2_F = mu_var_F * mu_var_F;
		if(mu_type == 0){
			mu2_R *= fixed_mu_R * fixed_mu_R;
			mu2_F *= fixed_mu_F * fixed_mu_F;
		}
		else{
			mu2_R *= s;
			mu2_F *= s;
		}
		double alphas = ch->get_alpha_s(mu2_R);
		
		double pth2; //=transversal Higgs momentum squared, check the cut for each contribution
		
		double fakecos;
		double pref_eta0 = phase3alt(1,xs[1],xs[2],xs[3],mz,mh,sqrs, p1,p2,E1,E3,cost1,cost2,cost12);
		pth2 = p1*p1*(1-cost1*cost1);
		if(use_pth_min && pth2 < pth_min*pth_min) pref_eta0 =0;
		double ampsqeta0 = NLO_soft_limit_ampsq(inv_mt_pow, p1,E1,cost1, sqrs, alphas);
		double pref_eta0y1 = phase3alt(1,xs[1],1,xs[3],mz,mh,sqrs, p1,p2,E1,E3,cost1,fakecos,cost12);
		pth2 = p1*p1*(1-cost1*cost1);
		if(use_pth_min && pth2 < pth_min*pth_min) pref_eta0y1 =0;
		double pref_eta0ym1 = phase3alt(1,xs[1],0,xs[3],mz,mh,sqrs, p1,p2,E1,E3,cost1,fakecos,cost12);
		pth2 = p1*p1*(1-cost1*cost1);
		if(use_pth_min && pth2 < pth_min*pth_min) pref_eta0ym1 =0;
		//ampsqeta0 == ampsqeta0y1 == ampsqeta0ym1, since soft limit is already independent of theta_gluon
		total -= get_pdfs(mu2_F)*ampsqeta0*((pref_eta0-pref_eta0y1)/(1-cost2) + (pref_eta0-pref_eta0ym1)/(1+cost2));
	}
	
	
	//Averaging factor for 2 incoming gluons. 256 = 2^2 (polarisation) * 8^2 (colour)
	//Flux factor = 1/2s
	total /= 256*2*s;
	
	//Divide by eta for the plus distribution and by 2*eta 
	total /= 2*eta*eta;
	
	
	//Compare with vh@nnlo
	/*cout << "ME full: " << ME2test << endl;
	cout << "dipole: " << ME2test-total << endl;
	cout << total << endl;
	exit(0);*/
	
	return total/pow(mt,inv_mt_pow);
	
}


double NLO_phase3_0(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase3_alt(xs, sqrs, get_pdfs, ch, 0);
}

double NLO_phase3_2(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase3_alt(xs, sqrs, get_pdfs, ch, 2);
}

double NLO_phase3_4(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase3_alt(xs, sqrs, get_pdfs, ch, 4);
}

double NLO_phase3_6(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase3_alt(xs, sqrs, get_pdfs, ch, 6);
}

double NLO_phase3_8(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	return NLO_phase3_alt(xs, sqrs, get_pdfs, ch, 8);
}

#include "../src/undef_params.h"
