#include "NLO_reducible.h"
#include <cmath>
#include "phase2.h"
#include "shared.h"
#include <iostream>
#include "../src/params.h"
#include "../src/def_params.h"
#include "clooptools.h"

double z2 = 1.6449340668482264365;

double NLO_reducible(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch, unsigned int inv_mt_pow){
                switch(inv_mt_pow){
                case 0:
                {
                        return NLO_reducible_0(xs, sqrs, get_pdfs, ch);
                }
                case 2:
                {
                        return NLO_reducible_2(xs, sqrs, get_pdfs, ch);
                }
                case 4:
                {
                        return NLO_reducible_4(xs, sqrs, get_pdfs, ch);
                }
                case 6:
                {
                        return NLO_reducible_6(xs, sqrs, get_pdfs, ch);
                }
                case 8:
                {
                        return NLO_reducible_8(xs, sqrs, get_pdfs, ch);
                }
                default:
                        cerr << "No NLO reducible amplitude squared defined for order 1/m_t^ " << inv_mt_pow << "." << endl;
                        exit(1);
        }
        return 0;
}


double NLO_reducible_0(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	double shat = sqrs*sqrs;
	if(use_inv_mass_cutoff && shat > inv_mass_cutoff*inv_mass_cutoff) return 0;
	double phasepref = phase2prefactor(shat, mz, mh);
	if(phasepref == 0) return 0;
	phasepref *= 2; //Factor from measure, dcos -> 2*dx
	double mz2 = mz*mz;
	double mh2 = mh*mh;
	//Set mu^2 = s = (p_Z+p_H)^2 or fixed value
	double mu2_R = mu_var_R * mu_var_R, mu2_F = mu_var_F * mu_var_F;
	if(mu_type == 0){
		mu2_R *= fixed_mu_R * fixed_mu_R;
		mu2_F *= fixed_mu_F * fixed_mu_F;
	}
	else{
		mu2_R *= shat;
		mu2_F *= shat;
	}
	double alphas = ch->get_alpha_s(mu2_R);
	double globalpref = alphas*alphas*alphas/Pi/Pi/Pi*alpha*alpha*Pi*Pi/768.0/sw2/sw2/cw2/cw2/mz2/mz2;
	double that =-(shat-mh2-mz2)/2+sqrt(kaellen_function(shat,mh2,mz2))*(xs[0]-1.0/2.0);
	double uhat =-(shat-mz2-mh2)/2-sqrt(kaellen_function(shat,mh2,mz2))*(xs[0]-1.0/2.0);
	double flux = 1./2./shat;
	//Check cut on Higgs transversal momentum
	if(use_pth_min){
		double pth = sqrt(1/shat*(that-mz2)*(uhat-mz2)-mz2);
		if(pth < pth_min) return 0;
	}
	double ampsq = flux*phasepref*globalpref
		*(
		  shat * (shat - mh2 - mz2)
		*(-2.0 + log(-uhat/mz2)*mz2/(uhat-mz2) + log(-that/mz2)*mz2/(that-mz2))
		+ shat * (mh2 - mz2 + shat)
		*(-mz2/(that-mz2) - mz2/(uhat-mz2) + log(-uhat/mz2)*mz2*mz2/(uhat-mz2)/(uhat-mz2)+log(-that/mz2)*mz2*mz2/(that-mz2)/(that-mz2)
		)
		);

	double GF = sqrt(2.)*Pi*alpha/2./mz2/cw2/sw2;
	double NLOamplsq0;
	#include "NLO_reducible_0.ampsq"
	NLOamplsq0 *= flux*phasepref;

	return ampsq * get_pdfs(mu2_F);
}

double NLO_reducible_2(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	double shat = sqrs*sqrs;
	if(use_inv_mass_cutoff && shat > inv_mass_cutoff*inv_mass_cutoff) return 0;
	double phasepref = phase2prefactor(shat, mz, mh);
	if(phasepref == 0) return 0;
	phasepref *= 2; //Factor from measure, dcos -> 2*dx
	double mz2 = mz*mz;
	double mh2 = mh*mh;
	//Set mu^2 = s = (p_Z+p_H)^2 or fixed value
	double mu2_R = mu_var_R * mu_var_R, mu2_F = mu_var_F * mu_var_F;
	if(mu_type == 0){
		mu2_R *= fixed_mu_R * fixed_mu_R;
		mu2_F *= fixed_mu_F * fixed_mu_F;
	}
	else{
		mu2_R *= shat;
		mu2_F *= shat;
	}
	double alphas = ch->get_alpha_s(mu2_R);
	double globalpref = alphas*alphas*alphas/Pi/Pi/Pi*alpha*alpha*Pi*Pi/768.0/sw2/sw2/cw2/cw2/mz2/mz2;
	double that =-(shat-mh2-mz2)/2+sqrt(kaellen_function(shat,mh2,mz2))*(xs[0]-1.0/2.0);
	double uhat =-(shat-mz2-mh2)/2-sqrt(kaellen_function(shat,mh2,mz2))*(xs[0]-1.0/2.0);
	double flux = 1./2./shat;
	//Check cut on Higgs transversal momentum
	if(use_pth_min){
		double pth = sqrt(1/shat*(that-mz2)*(uhat-mz2)-mz2);
		if(pth < pth_min) return 0;
	}

	double GF = sqrt(2.)*Pi*alpha/2./mz2/cw2/sw2;
	double NLOamplsq2;
	#include "NLO_reducible_2.ampsq"
	NLOamplsq2 *= flux*phasepref;

	return NLOamplsq2 * get_pdfs(mu2_F);
}

double NLO_reducible_4(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	double shat = sqrs*sqrs;
	if(use_inv_mass_cutoff && shat > inv_mass_cutoff*inv_mass_cutoff) return 0;
	double phasepref = phase2prefactor(shat, mz, mh);
	if(phasepref == 0) return 0;
	phasepref *= 2; //Factor from measure, dcos -> 2*dx
	double mz2 = mz*mz;
	double mh2 = mh*mh;
	//Set mu^2 = s = (p_Z+p_H)^2 or fixed value
	double mu2_R = mu_var_R * mu_var_R, mu2_F = mu_var_F * mu_var_F;
	if(mu_type == 0){
		mu2_R *= fixed_mu_R * fixed_mu_R;
		mu2_F *= fixed_mu_F * fixed_mu_F;
	}
	else{
		mu2_R *= shat;
		mu2_F *= shat;
	}
	double alphas = ch->get_alpha_s(mu2_R);
	double globalpref = alphas*alphas*alphas/Pi/Pi/Pi*alpha*alpha*Pi*Pi/768.0/sw2/sw2/cw2/cw2/mz2/mz2;
	double that =-(shat-mh2-mz2)/2+sqrt(kaellen_function(shat,mh2,mz2))*(xs[0]-1.0/2.0);
	double uhat =-(shat-mz2-mh2)/2-sqrt(kaellen_function(shat,mh2,mz2))*(xs[0]-1.0/2.0);
	double flux = 1./2./shat;
	//Check cut on Higgs transversal momentum
	if(use_pth_min){
		double pth = sqrt(1/shat*(that-mz2)*(uhat-mz2)-mz2);
		if(pth < pth_min) return 0;
	}

	double GF = sqrt(2.)*Pi*alpha/2./mz2/cw2/sw2;
	double NLOamplsq4;
	#include "NLO_reducible_4.ampsq"
	NLOamplsq4 *= flux*phasepref;

	return NLOamplsq4 * get_pdfs(mu2_F);
}

double NLO_reducible_6(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	double shat = sqrs*sqrs;
	if(use_inv_mass_cutoff && shat > inv_mass_cutoff*inv_mass_cutoff) return 0;
	double phasepref = phase2prefactor(shat, mz, mh);
	if(phasepref == 0) return 0;
	phasepref *= 2; //Factor from measure, dcos -> 2*dx
	double mz2 = mz*mz;
	double mh2 = mh*mh;
	//Set mu^2 = s = (p_Z+p_H)^2 or fixed value
	double mu2_R = mu_var_R * mu_var_R, mu2_F = mu_var_F * mu_var_F;
	if(mu_type == 0){
		mu2_R *= fixed_mu_R * fixed_mu_R;
		mu2_F *= fixed_mu_F * fixed_mu_F;
	}
	else{
		mu2_R *= shat;
		mu2_F *= shat;
	}
	double alphas = ch->get_alpha_s(mu2_R);
	double globalpref = alphas*alphas*alphas/Pi/Pi/Pi*alpha*alpha*Pi*Pi/768.0/sw2/sw2/cw2/cw2/mz2/mz2;
	double that =-(shat-mh2-mz2)/2+sqrt(kaellen_function(shat,mh2,mz2))*(xs[0]-1.0/2.0);
	double uhat =-(shat-mz2-mh2)/2-sqrt(kaellen_function(shat,mh2,mz2))*(xs[0]-1.0/2.0);
	double flux = 1./2./shat;
	//Check cut on Higgs transversal momentum
	if(use_pth_min){
		double pth = sqrt(1/shat*(that-mz2)*(uhat-mz2)-mz2);
		if(pth < pth_min) return 0;
	}

	double GF = sqrt(2.)*Pi*alpha/2./mz2/cw2/sw2;
	double NLOamplsq6;
	#include "NLO_reducible_6.ampsq"
	NLOamplsq6 *= flux*phasepref;

	return NLOamplsq6 * get_pdfs(mu2_F);
}

double NLO_reducible_8(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	double shat = sqrs*sqrs;
	if(use_inv_mass_cutoff && shat > inv_mass_cutoff*inv_mass_cutoff) return 0;
	double phasepref = phase2prefactor(shat, mz, mh);
	if(phasepref == 0) return 0;
	phasepref *= 2; //Factor from measure, dcos -> 2*dx
	double mz2 = mz*mz;
	double mh2 = mh*mh;
	//Set mu^2 = s = (p_Z+p_H)^2 or fixed value
	double mu2_R = mu_var_R * mu_var_R, mu2_F = mu_var_F * mu_var_F;
	if(mu_type == 0){
		mu2_R *= fixed_mu_R * fixed_mu_R;
		mu2_F *= fixed_mu_F * fixed_mu_F;
	}
	else{
		mu2_R *= shat;
		mu2_F *= shat;
	}
	double alphas = ch->get_alpha_s(mu2_R);
	double globalpref = alphas*alphas*alphas/Pi/Pi/Pi*alpha*alpha*Pi*Pi/768.0/sw2/sw2/cw2/cw2/mz2/mz2;
	double that =-(shat-mh2-mz2)/2+sqrt(kaellen_function(shat,mh2,mz2))*(xs[0]-1.0/2.0);
	double uhat =-(shat-mz2-mh2)/2-sqrt(kaellen_function(shat,mh2,mz2))*(xs[0]-1.0/2.0);
	double flux = 1./2./shat;
	//Check cut on Higgs transversal momentum
	if(use_pth_min){
		double pth = sqrt(1/shat*(that-mz2)*(uhat-mz2)-mz2);
		if(pth < pth_min) return 0;
	}

	double GF = sqrt(2.)*Pi*alpha/2./mz2/cw2/sw2;
	double NLOamplsq8;
	#include "NLO_reducible_8.ampsq"
	NLOamplsq8 *= flux*phasepref;

	return NLOamplsq8 * get_pdfs(mu2_F);
}

double NLO_reducible_exact(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	double shat = sqrs*sqrs;
	if(use_inv_mass_cutoff && shat > inv_mass_cutoff*inv_mass_cutoff) return 0;
	double phasepref = phase2prefactor(shat, mz, mh);
	if(phasepref == 0) return 0;
	phasepref *= 2; //Factor from measure, dcos -> 2*dx
	double mz2 = mz*mz;
	double mh2 = mh*mh;
	//Set mu^2 = s = (p_Z+p_H)^2 or fixed value
	double mu2_R = mu_var_R * mu_var_R, mu2_F = mu_var_F * mu_var_F;
	if(mu_type == 0){
		mu2_R *= fixed_mu_R * fixed_mu_R;
		mu2_F *= fixed_mu_F * fixed_mu_F;
	}
	else{
		mu2_R *= shat;
		mu2_F *= shat;
	}
	double alphas = ch->get_alpha_s(mu2_R);
	double globalpref = alphas*alphas*alphas/Pi/Pi/Pi*alpha*alpha*Pi*Pi/768.0/sw2/sw2/cw2/cw2/mz2/mz2;
	double that =-(shat-mh2-mz2)/2+sqrt(kaellen_function(shat,mh2,mz2))*(xs[0]-1.0/2.0);
	double uhat =-(shat-mz2-mh2)/2-sqrt(kaellen_function(shat,mh2,mz2))*(xs[0]-1.0/2.0);
	double flux = 1./2./shat;

	//Check cut on Higgs transversal momentum
	if(use_pth_min){
		double pth = sqrt(1/shat*(that-mz2)*(uhat-mz2)-mz2);
		if(pth < pth_min) return 0;
	}
	double ampsq = flux*phasepref*globalpref
		*(
		  shat * (shat - mh2 - mz2)
		*(-2.0 + log(-uhat/mz2)*mz2/(uhat-mz2) + log(-that/mz2)*mz2/(that-mz2))
		+ shat * (mh2 - mz2 + shat)
		*(-mz2/(that-mz2) - mz2/(uhat-mz2) + log(-uhat/mz2)*mz2*mz2/(uhat-mz2)/(uhat-mz2)+log(-that/mz2)*mz2*mz2/(that-mz2)/(that-mz2)
		)
		);

	double GF = sqrt(2.)*Pi*alpha/2./mz2/cw2/sw2;
	double kaellenroot = sqrt(kaellen_function(shat,mz2,mh2));
	double sp12 = shat/2.;
	double sp13 = (uhat - mz2)/2.;
	double sp23 = (that - mz2)/2.;

	double T;
	#include "NLO_reducible_exact.amplsq"
        clearcache();
	T *= flux*phasepref;

	return T * get_pdfs(mu2_F);
}


