#include <vector>
#include "LO.h"
#include "phase2.h"
#include "../src/params.h"
#include "../src/def_params.h"
#include "../src/channel.h"
#include "NLO_phase2.h"
#include "NLO_phase2eta.h"
#include "NLO_phase3.h"
#include "NLO_reducible.h"


inline double pade22(const std::vector<double> & c){
	std::vector<double> a(3),b(3);
	double tmp = c[2]*c[2]-c[1]*c[3];
	a[0] = c[0];
	if(tmp == 0){
		if(c[1] != 0){
			a[1] = (c[1]*c[1]-c[0]*c[2])/c[1];
			a[2] = 0;
			b[1] = c[2]/c[1];
			b[2] = 0;
		}
		else{
			a[1] = a[2] = b[1] = b[2] = 0;
		}	
	}
	else{
		a[1] = c[1] - c[0]*(c[2]*c[3]-c[1]*c[4])/tmp;
		a[2] = c[2] - (c[1]*c[2]*c[3]-c[0]*c[3]*c[3]-c[1]*c[1]*c[4]+c[0]*c[2]*c[4])/tmp;
		b[1] = (c[1]*c[4]-c[2]*c[3])/tmp;
		b[2] = (c[3]*c[3]-c[2]*c[4])/tmp;
	}
	double numer = 0, denom = 1;
	for(int i = 0; i < (int)a.size(); ++i) numer += a[i] * pow(mt,-2*i);
	for(int i = 1; i < (int)b.size(); ++i) denom += b[i] * pow(mt,-2*i);
	if(denom == 0) return 0;
	return numer/denom;
}


double pade22_LO(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	std::vector<double> c(5);
	for(unsigned int i = 0; i < c.size(); ++i) c[i] = LO(xs, sqrs, get_pdfs, ch, 2*i) * pow(mt,2*i);
	return pade22(c);
}


double pade22_NLO_phase2(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	std::vector<double> c(5);
	for(unsigned int i = 0; i < c.size(); ++i) c[i] = NLO_phase2(xs, sqrs, get_pdfs, ch, 2*i) * pow(mt,2*i);
	return pade22(c);
}

double pade22_NLO_phase2eta(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	std::vector<double> c(5);
	for(unsigned int i = 0; i < c.size(); ++i) c[i] = NLO_phase2eta(xs, sqrs, get_pdfs, ch, 2*i) * pow(mt,2*i);
	return pade22(c);
}

double pade22_NLO_phase3(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	std::vector<double> c(5);
	for(unsigned int i = 0; i < c.size(); ++i) c[i] = NLO_phase3_alt(xs, sqrs, get_pdfs, ch, 2*i) * pow(mt,2*i);
	return pade22(c);
}

double pade22_NLO_reducible(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	std::vector<double> c(5);
	for(unsigned int i = 0; i < c.size(); ++i) c[i] = NLO_reducible(xs, sqrs, get_pdfs, ch, 2*i) * pow(mt,2*i);
	return pade22(c);
}

double pade22_NLO_sum(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	std::vector<double> c(5);
	for(unsigned int i = 0; i < c.size(); ++i){
		c[i] = (NLO_phase3_alt(xs, sqrs, get_pdfs, ch, 2*i)
		      + NLO_phase2eta(xs, sqrs, get_pdfs, ch, 2*i)
		      + NLO_phase2(xs, sqrs, get_pdfs, ch, 2*i)) * pow(mt,2*i);
	 }
	return pade22(c);
}

double pade22_LO_NLO_sum(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch){
	std::vector<double> c(5);
	for(unsigned int i = 0; i < c.size(); ++i){
		c[i] = (NLO_phase3_alt(xs, sqrs, get_pdfs, ch, 2*i)
		      + NLO_phase2eta(xs, sqrs, get_pdfs, ch, 2*i)
		      + NLO_phase2(xs, sqrs, get_pdfs, ch, 2*i)
		      + LO(xs, sqrs, get_pdfs, ch, 2*i)) * pow(mt,2*i);
	 }
	return pade22(c);
}

























#include "../src/undef_params.h"
