#include <math.h>
#include "shared.h"
#include "phase2.h"
#include "../src/params.h"
#include "../src/def_params.h"

//2-particle phase space = this prefactor * integration over {-1,1,cos(theta)}*1
double phase2prefactor(double arg_s, double arg_m1, double arg_m2){
	double kaellen = kaellen_function(arg_s, arg_m1*arg_m1, arg_m2*arg_m2);
	if (arg_s < (arg_m1 + arg_m2)*(arg_m1 + arg_m2)) return 0;
	//	if(kaellen < 0) return 0;
	//	if(arg_s*arg_s - kaellen/2 < arg_s*(arg_m1+arg_m2)*(arg_m1+arg_m2)) return 0;
	return sqrt(kaellen)/16/Pi/arg_s;
}

//2-particle phase space prefactor as above, but one of the 2 incoming particles is scaled 
// by (1-eta). This means we are no longer in the center of mass system. For eta=0 this is
// equivalent to "phase2prefactor".

//Does not seem to yield correct results
double phase2prefactor_nonCMS(double arg_sqrs, double arg_m1, double arg_m2, double eta, double cos){
	double s = arg_sqrs*arg_sqrs;
	double kaellen = kaellen_function(s*(1-eta), arg_m1*arg_m1, arg_m2*arg_m2);
	double root_arg = kaellen - eta*eta*(1-cos*cos)*arg_m1*arg_m1*s;

	//root_arg needs to be positive for the momentum to be real
	if(root_arg < 0) return 0;

	double mom_abs = ((2.-eta)*sqrt(root_arg) - cos*eta*((1-eta)*s+arg_m1*arg_m1-arg_m2*arg_m2)) / arg_sqrs / ((2-eta)*(2-eta)-cos*cos*eta*eta);
	//absolute value of momentum needs to be positive
	if(mom_abs < 0) return 0;
	// ??? if this \theta function is not used the results seem to be OK ... ???


	if((1-eta)*s - 2*mom_abs*mom_abs - eta*cos*arg_sqrs*mom_abs < (arg_m1+arg_m2)*(arg_m1+arg_m2)) return 0;

	double en1 = sqrt(mom_abs*mom_abs + arg_m1*arg_m1);
	//double en2 = sqrt(s/4*eta*eta + eta*arg_sqrs*mom_abs*cos + mom_abs*mom_abs + arg_m2*arg_m2);
	//double tmp = fabs( en1 + en2 - (2-eta) * arg_sqrs / 2);

	return mom_abs*mom_abs / ((2.-eta)*mom_abs + eta*cos * en1) / Pi / 4. / arg_sqrs;
}

#include "../src/undef_params.h"
