
#ifndef PHASE3_H_
#define PHASE3_H_

#include <math.h>
#include "shared.h"

/*
 * 3-body phase integral for 2 massive and 1 massless particle in CMS frame, with phi2==0
 *
 * p1^mu = (E1, p1*sin(theta1)*cos(phi1), p1*sin(theta1)*sin(phi1), p1*cos(theta1))		mass^2 = m1^2
 * p2^mu = (p2, p2*sin(theta2), 0, p2*cos(theta2)										mass^2 = 0
 * p3^mu = (E3, -3vec(p1)-3vec(p2))														mass^2 = m3^2
 *
 * arg_sqrs = sqrt(s)
 * cost1 = cos(theta1)
 * cost2 = cos(theta2)
 * cost12 = cos(angle between p1 & p2)
 */
double phase3(double x0,double x1, double x2, double x3, double m1, double m3, double arg_sqrs,
		double & p1, double & p2, double & E1, double & E3, double & cost1, double & cost2, double & cost12);


double phase3alt(double x0, double x1, double x2, double x3, double m1, double m3, double arg_sqrs, double & p1, double & eta,
		double & E1, double & E3, double & cost1, double & cost2, double & cost12);

#endif /* PHASE3_H_ */
