

#include "channel.h"
#include <set>
#include "params.h"



//Read in user-defined channels
#include "def_params.h"
#include "../includes.h"
#include "undef_params.h"




map<string, channel> channel::all_channels;
vector<channel> channel::active_channels;


channel::channel(){
	pdf = nullptr;
	dropcount=nullptr;
}


void channel::read_in_channels(const configuration& cfg) {
	//Read in user-defined channels into x_section_channels
#include "../channels.h"

	//Get list of active channels from the config file
	vector<string> active = cfg.get_1d<string>("active channels");

	//Make sure all active channels are actually defined
	for(unsigned int i = 0; i < active.size(); ++i){
		if(!all_channels.count(active[i])){
			cerr << "Channel \"" << active[i] << "\" was declared active but never defined in channels.h." << endl;
			exit(1);
		}
		active_channels.push_back(all_channels[active[i]]);
	}


	//Print active channels
	cout << "Active channels: ";
	for(auto it = active_channels.begin(); it != active_channels.end();){
		cout << it->name;
		 ++it;
		if(it != active_channels.end()) cout << ", ";
	}
	cout << endl;
}

void channel::add_channel(const string & name, channel::PTYPE pdfs, unsigned int ndim, double (* cfun)(const double [], double, std::function<double(double)>, const channel *)) {
	channel ch;
	ch.name = name;
	ch.pdfs = pdfs;
	ch.ndim = ndim;
	ch.cfun = cfun;
	all_channels[name] = ch;
}

//Returns the value of alpha_s at scale^2
double channel::get_alpha_s(double scale2) const{
	return pdf->alphasQ2(scale2);
}
