#ifndef CHANNEL_H_
#define CHANNEL_H_


#include <atomic>
#include <string.h>
#include <map>
#include <functional>
#include "config.h"
#include "LHAPDF/LHAPDF.h"

using std::string;
using std::map;


class channel {
public:
	//-----------------------------------------
	// type of the collision
	// 1 -- pp interaction, -1 -- p-antip
	enum XTYPE {PP = 1, PB = -1};


	//-----------------------------------------
	// type of partonic interaction (for the proper PDFs)

	enum PTYPE {GG = 0, QG, QB, QQ, QP};



	//Members
	PTYPE pdfs; // type of PDF ids
	string name; // channel id
	// coefficient function: Returns partonic phase space point. 
		//xs = partonic phase space parameters
		//sqrs = sqrt(s_partonic)
		//get_pdfs = a function that takes the factorisation scale squared as a parameter and returns the product of the pdfs in the hadronic case or 1 in the partonic case
		//ch = a pointer to the channel object
	double (* cfun)(const double xs[], double sqrs, std::function<double(double)> get_pdfs, const channel * ch);
	unsigned int ndim; // integration dimension
	LHAPDF::PDF * pdf; //Non-owning pointer to PDF object
	std::atomic_uint * dropcount;
	

	channel();
	static void read_in_channels(const configuration & s);
	static void add_channel(const string & name, channel::PTYPE pdfs, unsigned int ndim, double (* cfun)(const double [], double sqrs, std::function<double(double)> get_pdfs, const channel * ch));
	static map<string, channel> all_channels;
	static vector<channel> active_channels;
	double get_alpha_s(double scale2) const; 
};





#endif /* CHANNEL_H_ */
