
#include "config.h"

#include <string>
#include <sstream>
#include <fstream>
#include <iostream>
#include <algorithm>







vector<string> configuration::parse_list(const string& s, const char * braces_and_sep) const{
	vector<string> ret;
	if(s.size() == 0) return ret;


	if(string(braces_and_sep).size() != 3){
		cerr << "Braces and separator string has incorrect length: " << braces_and_sep << endl;
		cerr << "Length should be 3: <brace open><brace close><separator>" << endl;
		exit(1);
	}
	const char open = braces_and_sep[0];
	const char close = braces_and_sep[1];
	//const char sep = braces_and_sep[2];



	int nest_level = 1;

	size_t pos1 = 0, pos2 = 0;

	if(s.at(0) == open) ++pos1;
	size_t start = pos1;

	while(true){
		if(nest_level < 1) break;
		pos2 = s.find_first_of(braces_and_sep, pos1);
		if(pos2 == string::npos){
			if(nest_level > 1){
				cerr << "Missing closing brace: " << s << endl;
				exit(1);
			}
			ret.push_back(s.substr(start));
			break;
		}
		pos1 = pos2+1;

		if(s.at(pos2) == open){
			++nest_level;
		}
		else if(s.at(pos2) == close){
			if(nest_level == 1){
				ret.push_back(s.substr(start, pos2-start));
			}
			--nest_level;
		}
		else{
			if(nest_level == 1){
				ret.push_back(s.substr(start, pos2-start));
				start = pos2+1;
			}
		}
	}

	return ret;
}




void configuration::read_in(bool must_exist) {
	//Clear old settings (if any)
	smap.clear();
	//Read in new settings
	ifstream f(inputfile.c_str());
	if(f.is_open()){
		string line;
		while(f.good()){
			getline(f,line);
			if(line.size() > 0 && (line.at(0) == '*' || line.at(0) == '#')) continue; //Allow for comments
			size_t pos = line.find(':');
			if(pos == string::npos) continue;
			string s = line.substr(pos+1);
			s.erase(std::remove_if(s.begin(), s.end(), ::isspace), s.end());
			smap[line.substr(0,pos)] = s;
		}
		f.close();
	}
	else if(must_exist){
		cerr << "File \"" << inputfile << "\" not found." << endl;
		exit(1);
	}
}

configuration::configuration(const string& input_file, bool must_exist) {
	inputfile = input_file;
	read_in(must_exist);
}

configuration::configuration(const char* input_file, bool must_exist) {
	inputfile = input_file;
	read_in(must_exist);
}

void configuration::change_input(const string& input_file, bool must_exist) {
	inputfile = input_file;
	read_in(must_exist);
}
