

#include "pdfs.h"
#include "LHAPDF/LHAPDF.h"
#include <map>
#include <memory>

using namespace std;


map<string, shared_ptr<LHAPDF::PDF> > all_pdfs;

LHAPDF::PDF * get_pdf(const std::string & pdf_name){
	if(!all_pdfs.count(pdf_name)) all_pdfs[pdf_name] = shared_ptr<LHAPDF::PDF>(LHAPDF::mkPDF(pdf_name.c_str(), 0));
	return all_pdfs[pdf_name].get();
}



double pdfs(const channel & ch, double x1, double x2, double Q){

	double r = 0;
	int flavours = 5; //Number of flavours. Could make this a parameter at some point.
	int xt = 1; //Collision type: 1 for proton-proton, -1 for proton-antiproton
	
	if(ch.pdf == nullptr){
		cerr << "PDF set not loaded for channel " << ch.name << endl;
		exit(1);
	}
	
	switch(ch.pdfs) { // Switch over parton types
	case channel::GG: // gluon-gluon
		return ch.pdf->xfxQ(0,x1,Q) * ch.pdf->xfxQ(0,x2,Q);

	case channel::QG: // quark-gluon
		for(int f = 1; f <= flavours; f++)
			r += ch.pdf->xfxQ(f,x1,Q) * ch.pdf->xfxQ(0,x2,Q) + ch.pdf->xfxQ(-f,x1,Q) * ch.pdf->xfxQ(0,x2,Q);
		return 2 * r; // = qg + gq

	case channel::QB: // quark-antiquark: p-p different from p-antip
		for(int f = 1; f <= flavours; f++)
			r += ch.pdf->xfxQ(f,x1,Q) * ch.pdf->xfxQ(-f * xt,x2,Q) + ch.pdf->xfxQ(-f,x1,Q) * ch.pdf->xfxQ(f * xt,x2,Q);
		return r;

	case channel::QQ: // quark-quark: p-p different from p-antip
		for(int f = 1; f <= flavours; f++)
			r += ch.pdf->xfxQ(f,x1,Q) * ch.pdf->xfxQ(f * xt,x2,Q) + ch.pdf->xfxQ(-f,x1,Q) * ch.pdf->xfxQ(-f * xt,x2,Q);
		return r;

	case channel::QP: // quark-quark prime
		for(int f1 = 1; f1 <= flavours; f1++)
			for(int f2 = f1 + 1; f2 <= flavours; f2++)
				r += (ch.pdf->xfxQ(f1,x1,Q) + ch.pdf->xfxQ(-f1,x1,Q)) * (ch.pdf->xfxQ(f2,x2,Q) + ch.pdf->xfxQ(-f2,x2,Q));
		return 2 * r; // symmetrize

	default:
		cerr << "pdf type unknown : " << ch.pdfs << endl;
		exit(1);
	}

	return 0;
}


double pdfs2(const channel & ch, double x1, double x2, double Q2){

	double r = 0;
	int flavours = 5; //Number of flavours. Could make this a parameter at some point.
	int xt = 1; //Collision type: 1 for proton-proton, -1 for proton-antiproton
	
	if(ch.pdf == nullptr){
		cerr << "PDF set not loaded for channel " << ch.name << endl;
		exit(1);
	}
	
	switch(ch.pdfs) { // Switch over parton types
	case channel::GG: // gluon-gluon
		return ch.pdf->xfxQ2(0,x1,Q2) * ch.pdf->xfxQ2(0,x2,Q2);

	case channel::QG: // quark-gluon
		for(int f = 1; f <= flavours; f++)
			r += ch.pdf->xfxQ2(f,x1,Q2) * ch.pdf->xfxQ2(0,x2,Q2) + ch.pdf->xfxQ2(-f,x1,Q2) * ch.pdf->xfxQ2(0,x2,Q2);
		return 2 * r; // = qg + gq

	case channel::QB: // quark-antiquark: p-p different from p-antip
		for(int f = 1; f <= flavours; f++)
			r += ch.pdf->xfxQ2(f,x1,Q2) * ch.pdf->xfxQ2(-f * xt,x2,Q2) + ch.pdf->xfxQ2(-f,x1,Q2) * ch.pdf->xfxQ2(f * xt,x2,Q2);
		return r;

	case channel::QQ: // quark-quark: p-p different from p-antip
		for(int f = 1; f <= flavours; f++)
			r += ch.pdf->xfxQ2(f,x1,Q2) * ch.pdf->xfxQ2(f * xt,x2,Q2) + ch.pdf->xfxQ2(-f,x1,Q2) * ch.pdf->xfxQ2(-f * xt,x2,Q2);
		return r;

	case channel::QP: // quark-quark prime
		for(int f1 = 1; f1 <= flavours; f1++)
			for(int f2 = f1 + 1; f2 <= flavours; f2++)
				r += (ch.pdf->xfxQ2(f1,x1,Q2) + ch.pdf->xfxQ2(-f1,x1,Q2)) * (ch.pdf->xfxQ2(f2,x2,Q2) + ch.pdf->xfxQ2(-f2,x2,Q2));
		return 2 * r; // symmetrize

	default:
		cerr << "pdf type unknown : " << ch.pdfs << endl;
		exit(1);
	}

	return 0;
}
