#ifndef PDFS_H_
#define PDFS_H_

#include <string>
#include "channel.h"
#include "LHAPDF/LHAPDF.h"
#include <map>
#include <memory>

using namespace std;

//-----------------------------------------
// parton luminosity functions for different in-states
// parameters:
// xt -- collider type (PP = 1 or PB = -1), effects QB and QQ channels
// pt -- type of the interaction (one of GG,QG,QB,QQ,QP)
// x1,x2 -- momentum fractions of partons (0.0..1.0)
// Q -- scale of the PDFs evaluation (in GeV, typically ~Mh)
double pdfs(const channel & ch, double x1, double x2, double Q);

// Same as pdfs, but with Q2 = (scale Q)^2 (to avoid costly calls to sqrt in some cases)
double pdfs2(const channel & ch, double x1, double x2, double Q2);


extern map<string, shared_ptr<LHAPDF::PDF> > all_pdfs;

LHAPDF::PDF * get_pdf(const std::string & pdf_name);



#endif /* PDFS_H_ */
